// Generated by LiveScript 1.5.0
var EventEmitter, uuid, wait, WebSocket, debug, WebSocketConnector;
EventEmitter = require('events').EventEmitter;
uuid = require('uuid');
wait = require('wait').wait;
WebSocket = require('ws');
debug = require('debug')('exorelay:websocket-listener');
WebSocketConnector = (function(superclass){
  var prototype = extend$((import$(WebSocketConnector, superclass).displayName = 'WebSocketConnector', WebSocketConnector), superclass).prototype, constructor = WebSocketConnector;
  function WebSocketConnector(arg$){
    var ref$;
    ref$ = arg$ != null
      ? arg$
      : {}, this.exocomHost = ref$.exocomHost, this.role = ref$.role, this.exocomPort = ref$.exocomPort;
    this._onSocketMessage = bind$(this, '_onSocketMessage', prototype);
    this._onSocketError = bind$(this, '_onSocketError', prototype);
    this._onSocketOpen = bind$(this, '_onSocketOpen', prototype);
    this._onSocketClose = bind$(this, '_onSocketClose', prototype);
    this.connect = bind$(this, 'connect', prototype);
    this.exocomPort = +this.exocomPort;
    this.lastSentMessage = null;
  }
  WebSocketConnector.prototype.close = function(done){
    if (!this.socket) {
      return done();
    }
    debug("no longer connected at 'ws://" + this.exocomHost + "/" + this.exocomPort + "'");
    this.shouldReconnectOnSocketClosed = false;
    switch (this.socket.readyState) {
    case WebSocket.CONNECTING:
      this.socket.terminate();
      return done();
    case WebSocket.OPEN:
      this.socket.on('close', done);
      return this.socket.close();
    case WebSocket.CLOSING:
      return this.socket.on('close', done);
    case WebSocket.CLOSED:
      return done();
    }
  };
  WebSocketConnector.prototype.replyMethodFor = function(activityId, sessionId){
    var this$ = this;
    switch (false) {
    case !!activityId:
      return this.emit('error', new Error('WebSocketConnector.replyMethodFor needs an activity-id'));
    }
    return function(messageName, payload){
      payload == null && (payload = {});
      return this$.send(messageName, payload, {
        activityId: activityId,
        sessionId: sessionId
      });
    };
  };
  WebSocketConnector.prototype.send = function(messageName, payload, options){
    var requestData;
    options == null && (options = {});
    switch (false) {
    case !!messageName:
      return this.emit('error', new Error('ExoRelay#send cannot send empty messages'));
    case typeof messageName === 'string':
      return this.emit('error', new Error('ExoRelay#send can only send string messages'));
    case typeof payload !== 'function':
      return this.emit('error', new Error('ExoRelay#send cannot send functions as payload'));
    }
    this._logSending(messageName, options);
    requestData = {
      name: messageName,
      sender: this.role,
      id: uuid.v1(),
      activityId: options.activityId || uuid.v1()
    };
    if (payload != null) {
      requestData.payload = payload;
    }
    if (options.sessionId) {
      requestData.sessionId = options.sessionId;
    }
    this.socket.send(JSON.stringify(requestData));
    return this.lastSentMessage = requestData;
  };
  WebSocketConnector.prototype.connect = function(){
    var x$;
    this.shouldReconnectOnSocketClosed = true;
    x$ = this.socket = new WebSocket("ws://" + this.exocomHost + ":" + this.exocomPort + "/services");
    x$.on('open', this._onSocketOpen);
    x$.on('message', this._onSocketMessage);
    x$.on('error', this._onSocketError);
    x$.on('close', this._onSocketClose);
    return x$;
  };
  WebSocketConnector.prototype._onSocketClose = function(){
    this.emit('offline');
    if (this.shouldReconnectOnSocketClosed) {
      return this.connect();
    }
  };
  WebSocketConnector.prototype._onSocketOpen = function(){
    return this.emit('online');
  };
  WebSocketConnector.prototype._onSocketError = function(error){
    switch (false) {
    case error.errno !== 'EADDRINUSE':
      return this.emit('error', "port " + this.exocomPort + " is already in use");
    default:
      return this.emit('error', error);
    }
  };
  WebSocketConnector.prototype._onSocketMessage = function(data){
    var requestData, result;
    requestData = this._parseRequest(
    JSON.parse(
    data));
    this._logReceived(requestData);
    switch (result = this.listeners('message')[0](requestData)) {
    case 'success':
      break;
    case 'missing message id':
      return this.emit('error', Error('missing message id'));
    case 'unknown message':
      return this.emit('error', Error("unknown message: '" + requestData.messageName + "'"));
    default:
      return this.emit('error', Error("unknown result code: '" + result + "'"));
    }
  };
  WebSocketConnector.prototype._logReceived = function(arg$){
    var messageName, id, activityId;
    messageName = arg$.messageName, id = arg$.id, activityId = arg$.activityId;
    return debug("received message '" + messageName + "' with id '" + id + "' in discussion of '" + activityId + "'");
  };
  WebSocketConnector.prototype._logSending = function(messageName, options){
    return debug("sending message '" + messageName + "' in discussion of '" + options.activityId + "'");
  };
  WebSocketConnector.prototype._parseRequest = function(req){
    return {
      messageName: req.name,
      payload: req.payload,
      activityId: req.activityId,
      id: req.id,
      sessionId: req.sessionId
    };
  };
  return WebSocketConnector;
}(EventEmitter));
module.exports = WebSocketConnector;
function bind$(obj, key, target){
  return function(){ return (target || obj)[key].apply(obj, arguments) };
}
function extend$(sub, sup){
  function fun(){} fun.prototype = (sub.superclass = sup).prototype;
  (sub.prototype = new fun).constructor = sub;
  if (typeof sup.extended == 'function') sup.extended(sub);
  return sub;
}
function import$(obj, src){
  var own = {}.hasOwnProperty;
  for (var key in src) if (own.call(src, key)) obj[key] = src[key];
  return obj;
}